/*
 * Copyright (c) 2012, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include <linux/input.h>
#include <linux/joystick.h>

#include "joystick.h"

int joystick_init(const char *devname, struct joystick_info *joyinfo)
{
	int fd;

	memset(joyinfo, 0, sizeof(*joyinfo));
	joyinfo->nb_axis = 2;
	joyinfo->nb_buttons = 2;
	joyinfo->version = 0x000800;
	strcpy(joyinfo->name, "Unknown");

	if ((fd = open(devname, O_RDONLY)) < 0) {
		perror("jstest");
		return -1;
	}

	ioctl(fd, JSIOCGVERSION, &joyinfo->version);
	ioctl(fd, JSIOCGAXES, &joyinfo->nb_axis);
	ioctl(fd, JSIOCGBUTTONS, &joyinfo->nb_buttons);
	ioctl(fd, JSIOCGNAME(JOYSTICK_NAME_LENGTH), joyinfo->name);
	ioctl(fd, JSIOCGAXMAP, joyinfo->axmap);
	ioctl(fd, JSIOCGBTNMAP, joyinfo->btnmap);

	joyinfo->fd = fd;
	return 0;
}

void joystick_dump(struct joystick_info *joyinfo)
{
	int i;

	if (joyinfo->nb_axis) {
		printf("Axes: ");
		for (i = 0; i < joyinfo->nb_axis; i++)
			printf("%2d:%6d ", i, joyinfo->axis[i]);
	}

	if (joyinfo->nb_buttons) {
		printf("Buttons: ");
		for (i = 0; i < joyinfo->nb_buttons; i++)
			printf("%2d:%s ", i, joyinfo->buttons[i] ? "on " : "off");
	}

	printf("\n");

}

void joystick_input(int fd, short event, void *arg)
{
	struct joystick_info *joyinfo;
	struct js_event js;

	joyinfo = arg;

	if (read(fd, &js, sizeof(struct js_event)) != sizeof(struct js_event)) {
		perror("\njstest: error reading");
		return;
	}

	switch(js.type & ~JS_EVENT_INIT) {
		case JS_EVENT_BUTTON:
			if (js.number >= joyinfo->nb_buttons) {
				fprintf(stderr,
					"\njstest: button number too high");
				return;
			}
			joyinfo->buttons[js.number] = js.value;
			break;
		case JS_EVENT_AXIS:
			if (js.number >= joyinfo->nb_axis) {
				fprintf(stderr,
					"\njstest: axe number too high");
				return;
			}
			joyinfo->axis[js.number] = js.value;
			break;
	}
}
