/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>
#include <netinet/in.h>

#include <cmdline_parse_ipaddr.h>
#include <cmdline_parse.h>

#include "xbee_atcmd.h"
#include "parse_atcmd.h"

/* This file is an example of extension of libcmdline. It provides an
 * example of named objects stored in a list, supporting the
 * completion on objects name */

static int
parse_atcmd(cmdline_parse_token_hdr_t *tk, const char *buf, void *res,
	    unsigned ressize)
{
	struct token_atcmd *tk2 = (struct token_atcmd *)tk;
	struct token_atcmd_data *tkd = &tk2->atcmd_data;
	struct xbee_atcmd *cmd;

	if (res && ressize < sizeof(struct object *))
		return -1;

	/* XXX should be related to an xbee device */
	cmd = xbee_atcmd_lookup_desc(buf);
	if (cmd == NULL)
		return -1;

	/* command must match flags */
	if ((cmd->flags & tkd->atcmd_mask) != tkd->atcmd_flags)
		return -1;

	/* store the address of object in structure */
	if (res)
		*(struct xbee_atcmd **)res = cmd;

	return 0;
}

static int
complete_atcmd_start(cmdline_parse_token_hdr_t *tk,
		     __attribute__((unused)) const char *tokstr,
		     void **opaque)
{
	struct xbee_atcmd *cmd;

	cmd = &xbee_atcmd_list[0];
	*opaque = (void *)cmd;
	return 0;
}

static int
complete_atcmd_iterate(cmdline_parse_token_hdr_t *tk, void **opaque,
		       char *dstbuf, unsigned int size)
{
	struct token_atcmd *tk2 = (struct token_atcmd *)tk;
	struct token_atcmd_data *tkd = &tk2->atcmd_data;
	struct xbee_atcmd *cmd;
	int len;

	cmd = *opaque;

	while (1) {
		if (cmd->name == NULL)
			return -1;

		/* skip commands that don't match flags */
		if ((cmd->flags & tkd->atcmd_mask) != tkd->atcmd_flags) {
			cmd++;
			continue;
		}

		len = snprintf(dstbuf, size, "%s", cmd->desc);
		if (len < 0 || len >= size)
			return -1;

		strcpy(dstbuf, cmd->desc);
		break;
	}
	cmd++;
	*opaque = cmd;
	return 0;
}

static int
cmdline_help_atcmd(cmdline_parse_token_hdr_t *tk, char *dstbuf,
		   unsigned int size)
{
	snprintf(dstbuf, size, "ATCMD");
	return 0;
}

struct cmdline_token_ops token_atcmd_ops = {
	.parse = parse_atcmd,
	.complete_start = complete_atcmd_start,
	.complete_iterate = complete_atcmd_iterate,
	.complete_end = NULL,
	.help = cmdline_help_atcmd,
};
