/*
 * Copyright (c) 2009, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>
#include <netinet/in.h>

#include <cmdline_parse_ipaddr.h>
#include <cmdline_parse.h>

#include "parse_monitor.h"

static int
parse_monitor(cmdline_parse_token_hdr_t *tk, const char *buf, void *res,
	       unsigned ressize)
{
	struct token_monitor *tk2 = (struct token_monitor *)tk;
	struct token_monitor_data *tkd = &tk2->monitor_data;
	struct monitor_reg *m;

	if (res && ressize < sizeof(struct monitor_reg *))
		return -1;

	LIST_FOREACH(m, tkd->list, next) {
		if (strcmp(buf, m->desc))
			continue;
		break;
	}
	if (m == NULL) /* not found */
		return -1;

	/* store the address of object in structure */
	if (res)
		*(struct monitor_reg **)res = m;

	return 0;
}

static int
complete_monitor_start(cmdline_parse_token_hdr_t *tk,
			__attribute__((unused)) const char *tokstr,
			void **opaque)
{
	struct token_monitor *tk2 = (struct token_monitor *)tk;
	struct token_monitor_data *tkd = &tk2->monitor_data;
	struct monitor_reg *m;

	m = LIST_FIRST(tkd->list);
	*opaque = (void *)m;
	if (m == NULL)
		return -1; /* no completion */
	return 0;
}

static int
complete_monitor_iterate(cmdline_parse_token_hdr_t *tk, void **opaque,
			  char *dstbuf, unsigned int size)
{
	struct monitor_reg *m;
	int len;

	m = *opaque;
	if (m == NULL)
		return -1;

	len = snprintf(dstbuf, size, "%s", m->desc);
	if (len < 0 || len >= size)
		return -1;

	strcpy(dstbuf, m->desc);
	m = LIST_NEXT(m, next);
	*opaque = m;
	return 0;
}


static int
cmdline_help_monitor(cmdline_parse_token_hdr_t *tk, char *dstbuf,
		      unsigned int size)
{
	snprintf(dstbuf, size, "Monitor-register");
	return 0;
}

struct cmdline_token_ops token_monitor_ops = {
	.parse = parse_monitor,
	.complete_start = complete_monitor_start,
	.complete_iterate = complete_monitor_iterate,
	.complete_end = NULL,
	.help = cmdline_help_monitor,
};
