/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>
#include <sys/queue.h>

#include <cmdline_parse.h>

#include "xbee_neighbor.h"
#include "xbee_atcmd.h"
#include "xbee_stats.h"
#include "xbee_buf.h"
#include "xbee_proto.h"
#include "xbee.h"

#include "parse_neighbor.h"

static int
parse_neighbor(cmdline_parse_token_hdr_t *tk, const char *buf, void *res,
	       unsigned ressize)
{
	struct token_neighbor *tk2 = (struct token_neighbor *)tk;
	struct token_neighbor_data *tkd = &tk2->neighbor_data;
	struct xbee_dev *dev = *tkd->xbee_dev;
	struct xbee_neigh *neigh;

	if (res && ressize < sizeof(struct xbee_neigh *))
		return -1;

	neigh = xbee_neigh_lookup(dev, buf);
	if (neigh == NULL) /* not found */
		return -1;

	/* store the address of xbee_neigh in structure */
	if (res)
		*(struct xbee_neigh **)res = neigh;

	return 0;
}

static int
complete_neighbor_start(cmdline_parse_token_hdr_t *tk,
			__attribute__((unused)) const char *tokstr,
			void **opaque)
{
	struct token_neighbor *tk2 = (struct token_neighbor *)tk;
	struct token_neighbor_data *tkd = &tk2->neighbor_data;
	struct xbee_dev *dev = *tkd->xbee_dev;
	struct xbee_neigh *neigh;

	neigh = LIST_FIRST(&dev->neigh_list);
	*opaque = (void *)neigh;
	if (neigh == NULL)
		return -1; /* no completion */
	return 0;
}

static int
complete_neighbor_iterate(cmdline_parse_token_hdr_t *tk, void **opaque,
			  char *dstbuf, unsigned int size)
{
	struct xbee_neigh *neigh;
	int len;

	neigh = *opaque;
	if (neigh == NULL)
		return -1;

	len = snprintf(dstbuf, size, "%s", neigh->name);
	if (len < 0 || len >= size)
		return -1;

	strcpy(dstbuf, neigh->name);
	neigh = LIST_NEXT(neigh, next);
	*opaque = neigh;
	return 0;
}


static int
cmdline_help_neighbor(cmdline_parse_token_hdr_t *tk, char *dstbuf,
		      unsigned int size)
{
	snprintf(dstbuf, size, "Neighbor");
	return 0;
}

struct cmdline_token_ops token_neighbor_ops = {
	.parse = parse_neighbor,
	.complete_start = complete_neighbor_start,
	.complete_iterate = complete_neighbor_iterate,
	.complete_end = NULL,
	.help = cmdline_help_neighbor,
};
