/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Callback when receiving data on a specific channel. The arguments
 * of the function are the xbee device, the channel ID, the type of
 * the frame (example: XBEE_TYPE_ATRESP), the pointer to the frame,
 * the length of the frame, and an opaque pointer (reserved for user) */
typedef void (xbee_rx_cb_t)(struct xbee_dev *, int, int, void *,
			    unsigned, void *);

/* an xbee queue */
struct xbee_channel {
	int registered;
	xbee_rx_cb_t *rx_cb;
	void *arg;
};

#define XBEE_DEFAULT_CHANNEL 0
#define XBEE_MAX_CHANNEL 16
#define XBEE_CHANNEL_ANY XBEE_MAX_CHANNEL

/* structure identifying a xbee device */
struct xbee_dev {
	const char *name;
	int fd;
	unsigned baud;
	struct xbee_bufq queue;
	struct xbee_channel channel[XBEE_MAX_CHANNEL];
	struct xbee_stats stats;
	struct xbee_neigh_list neigh_list;
};

/* initialize xbee library */
int xbee_init(void);

/* open an xbee device */
struct xbee_dev *xbee_open(const char *devname, unsigned baudrate);

/* closes an xbee device */
int xbee_close(struct xbee_dev *dev);

/* Register a channel, return the ID of the channel or a negative
 * value on error. The rx_cb is a pointer to a function that will be
 * called by xbee_read() when a frame is received for this channel. If
 * rx_cb is NULL, no callback will occur. The "channel" argument can
 * be XBEE_CHANNEL_ANY to let the library choose the channel, or a
 * channel number to request a specific one. */
int xbee_register_channel(struct xbee_dev *dev, int channel,
			  xbee_rx_cb_t *rx_cb, void *opaque);

/* Unregister a channel, return 0 on success */
int xbee_unregister_channel(struct xbee_dev *dev, int channel_id);

/* read data from device fd and put it in queue */
int xbee_read(struct xbee_dev *dev);

/* process all data in queue */
int xbee_process_queue(struct xbee_dev *dev);
