/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include <stdlib.h>

#include "xbee_atcmd.h"

struct xbee_atcmd xbee_atcmd_list[] = {
	{
		"WR",
		"write-param",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Write parameter values to non-volatile memory.",
	},
	{
		"RE",
		"restore-defaults",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Restore module parameters to factory defaults.",
	},
	{
		"FR",
		"soft-reset",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Software Reset. Responds with 'OK' then performs a "
		"reset 100ms later.",
	},
	{
		"AC",
		"apply-changes",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Apply Changes without exiting command mode.",
	},
	{
		"R1",
		"restore-compiled",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Restore module parameters to compiled defaults.",
	},
	{
		"VL",
		"version-long",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Shows detailed version information including"
		"application build date and time.",
	},
	{
		"DH",
		"dst-addr-high",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Upper 32 bits of the 64-bit destination address (0 "
		"to 0xFFFFFFFF, default is 0x0000FFFF).",
	},
	{
		"DL",
		"dst-addr-low",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Lower 32 bits of the 64-bit destination address (0 "
		"to 0xFFFFFFFF, default is 0x0000FFFF).",
	},
	{
		"DD",
		"device-type-id",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ,
		"Device Type Identifier, it can be used to differentiate "
		"multiple XBee-based products (0 to 0xFFFFFFFF, read-only, "
		"default is 0x40000).",
	},
	{
		"SH",
		"src-addr-high",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ,
		"Upper 32 bits of the 64-bit source address (read-only).",
	},
	{
		"SL",
		"src-addr-low",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ,
		"Lower 32 bits of the 64-bit source address (read-only).",
	},
	{
		"SE",
		"src-endpoint",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"The application source endpoint for all data transmissions "
		"(0 to 0xFF, default is 0xE8).",
	},
	{
		"DE",
		"dst-endpoint",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"The application destination endpoint for all data "
		"transmissions (0 to 0xFF, default is 0xE8).",
	},
	{
		"CI",
		"cluster-id",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Cluster Identifier for all data transmissions (0 to 0xFFFF, "
		"default is 0x11).",
	},
	{
		"NP",
		"max-rf-payload",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Maximum RF Payload Bytes that can be sent in a unicast "
		"transmission based on the current configuration (0 to "
		"0xFFFF).",
	},
	{
		"CE",
		"coord-end-device",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Coordinator/End Device, messaging mode of the module "
		"(0 - Normal, 1 - Indirect coordinator, 2 - Polling, default "
		"is 0).",
	},
	{
		"AP",
		"api-mode",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"API mode (0 - off, 1 - on, 2 - on with escape sequences).",
	},
	{
		"AO",
		"api-output-format",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"API Output Format (0 - standard [0x90 for RX], 1 - explicit "
		"addressing [0x91 for RX]).",
	},
	{
		"BD",
		"baud-rate",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Baud rate of serial interface (0-8 select preset standard "
		"rates, and 0x39 to 0x1c9c38 select baud rate).",
	},
	{
		"RO",
		"packetization-timeout",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Packetization Timeout: the inter-character silence required "
		"before packetization specified in character times (0 to 0xFF, "
		"default is 3).",
	},
	{
		"FT",
		"flow-control-thres",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Flow Control Threshhold. De-assert CTS and/or send XOFF when "
		"FT bytes are in the UART receive buffer. Re-assert CTS when "
		"less than FT - 16 bytes are in the UART receive buffer (0x11 "
		"to 0xEE, default is 0xBE).",
	},
	{
		"NB",
		"parity",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Parity (0 - no parity, 1 - even parity, 2 - odd parity, 3 - "
		"forced high parity, 4 - forced low parity). Default is 0.",
	},
	{
		"D7",
		"dio7",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO7 Configuration (0 - unmonitored input, 1 - CTS, 3 - "
		"digital input, 4 - digital output low, 5 - digital output "
		"high, 6 - RS-485 low Tx, 7 -  RS-485 high Tx). Default is "
		"0.",
	},
	{
		"D6",
		"dio6",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO6 Configuration (0 - unmonitored input, 1 - RTS, 3 - "
		"digital input, 4 - digital output low, 5 - digital output "
		"high). Default is 0.",
	},
	{
		"P0",
		"dio10-pwm0",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO10/PWM0 Configuration. (0 - unmonitored input, 1 - RSSI, 2 "
		"- PWM0, 3 - digital input, 4 - digital output low, 5 - "
		"digital output high). Default is 1.",
	},
	{
		"P1",
		"dio11-pwm1",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO11/PWM1 Configuration. (0 - unmonitored input, 2 "
		"- PWM1, 3 - digital input, 4 - digital output low, 5 - "
		"digital output high). Default is 0.",
	},
	{
		"P2",
		"dio12",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO12 Configuration. (0 - unmonitored input, "
		"3 - digital input, 4 - digital output low, 5 - "
		"digital output high). Default is 0.",
	},
	{
		"RP",
		"rssi-pwm",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Time RSSI signal will be output after last transmission. "
		"When RP = 0xFF, output will always be on (0 - 0xFF, default "
		"is 0x28 = 4 seconds).",
	},
	{
		"1S",
		"sensor-sample",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Forces a sample to be taken on an XBee Sensor device.",
	},
	{
		"D0",
		"dio0-ad0",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD0/DIO0 Configuration. (0 - unmonitored input, 1 - "
		"commission button enabled, 2 - analog input, 3 - digital "
		"input, 4 - digital output low, 5 - digital output high). "
		"Default is 1.",
	},
	{
		"D1",
		"dio1-ad1",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD1/DIO1 Configuration. (0 - unmonitored input, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 0.",
	},
	{
		"D2",
		"dio2-ad2",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD2/DIO2 Configuration. (0 - unmonitored input, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 0.",
	},
	{
		"D3",
		"dio3-ad3",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD3/DIO3 Configuration. (0 - unmonitored input, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 0.",
	},
	{
		"D4",
		"dio4-ad4",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD4/DIO4 Configuration. (0 - unmonitored input, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 0.",
	},
	{
		"D5",
		"dio5-ad5",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"AD4/DIO4 Configuration. (0 - unmonitored input, 1 - LED, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 1.",
	},
	{
		"D8",
		"dio8-sleep-rq",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO8/SLEEP_RQ Configuration. (0 - unmonitored input, 1 - LED, "
		"2 - analog input, 3 - digital input, 4 - digital output "
		"low, 5 - digital output high). Default is 0. When used as "
		"SLEEP_RQ, the D8 parameter should be configured in mode 0 "
		"or 3.",
	},
	{
		"D9",
		"dio9-on-sleep",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"DIO9/ON_SLEEP Configuration. (0 - unmonitored input, 1 - "
		"ON/SLEEP, 2 - analog input, 3 - digital input, 4 - digital "
		"output low, 5 - digital output high). Default is ?.",
	},
	{
		"PR",
		"pull-up-resistor",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Pull-up Resistor. Bit field that configures the internal "
		"pull-up resistors for the I/O lines (bit set = pull-up "
		"enabled). Range is from 0 to 0x1FFF, default is 0x1FFF.",
	},
	{
		"M0",
		"pwm0-out-level",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"PWM0 Output Level. The line should be configured as a PWM "
		"output using the P0 command (0 to 0x3FF, default is 0).",
	},
	{
		"M1",
		"pwm1-out-level",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"PWM1 Output Level. The line should be configured as a PWM "
		"output using the P1 command (0 to 0x3FF, default is 0).",
	},
	{
		"LT",
		"led-blink-time",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Associate LED Blink Time (should be enabled through D5 "
		"command). Range is 0x14-0xFF (x10ms), default is 0.",	},
	{
		"IS",
		"force-sample",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Forces a read of all enabled digital and "
		"analog input lines.",
	},
	{
		"IC",
		"digital-change-detect",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"I/O Digital Change Detection. If a pin is enabled as a "
		"digital input/output, the IC command can be used to "
		"force an immediate I/O sample transmission when the DIO "
		"state changes. IC is a bitmask, range is 0 to 0xFFFF, "
		"default is 0",
	},
	{
		"IR",
		"io-sample-rate",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"IO Sample Rate for periodic sampling. If zero, periodic "
		"sampling is disabled. Else the value is in milliseconds "
		"(range 0 to 0xFFFF, default is 0).",
	},
	{
		"CB",
		"comissioning-button",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Commissioning Pushbutton, simulate commissioning button "
		"in software. The parameter value should be set to the number "
		"of button presses to be simulated (range is 0 to 4).",
	},
	{
		"VR",
		"firmware-version",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ,
		"Firmware version of the module (read only).",
	},
	{
		"HV",
		"hardware-version",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Hardware version of the module (read only)."
	},
	{
		"CK",
		"config-code",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ,
		"Configuration Code, that can be used as a quick "
		"check to determine if a node has been configured as "
		"desired (read-only, 0-0xFFFFFFFF)."
	},
	{
		"ER",
		"rf-errors",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Number of times a packet was received which contained errors "
		"of some sort. Read-only, saturate at 0xFFFF.",
	},
	{
		"GD",
		"good-packets",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Number of good received frames. Read-only, saturate at "
		"0xFFFF.",
	},
	{
		"RP",
		"rssi-pwm-timer",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"RSSI PWM timer, the time in tenth of seconds that the RSSI "
		"output indicating signal strength will remain active after "
		"the last reception (1 to 0xff, default is 0x20 = 3.2 secs).",
	},
	{
		"TR",
		"tx-errors",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Transmission Errors, the number of MAC frames that "
		"exhaust MAC retries without ever receiving a MAC "
		"acknowledgement message. Read-only, saturate at 0xFFFF.",
	},
	{
		"TP",
		"temperature",
		XBEE_ATCMD_F_PARAM_S16 | XBEE_ATCMD_F_READ,
		"Temperature. Read module temperature in (tenths of ?) "
		"Celsius. Negatives temperatures can be returned (read-only, "
		"from 0xff74 [-140] to 0x0258 [600]).",
	},
	{
		"DB",
		"rx-signal-strength",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Received Signal Strength of the last received RF data "
		"packet measured in -dBm. For example if DB returns 0x60, "
		"then the RSSI of the last packet received was -96dBm "
		"(read-only)."
	},
	{
		"DC",
		"duty-cycle",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ,
		"Duty Cycle. Returns a current usage percentage of the "
		"10% duty cycle measured over the period of 1 hour "
		"(read-only, from 0 to 0x64).",
	},
	{
		"RC",
		"rssi-for-channel", //XXX in fact it is a read with a param
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_WRITE,
		"Reads the dBm level (RSSI) of the designated "
		"channel.",
	},
	{
		"R#",
		"reset-number",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ,
		"Tells the reason for the last module reset (0 - Power up "
		"reset, 2 - Watchdog reset, 3 - Software reset, 4 - Reset "
		"line reset, 5 - Brownout reset). Read-only.",
	},
	{
		"TA",
		"tx-ack-errors",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ,
		"Transmit Acknowlegement Errors. Incremented once for "
		"each failed ack retry (read-only, from 0 to 0xFFFF).",
	},
	{
		"%V",
		"supply-voltage",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ,
		"Voltage on the Vcc pin in mV (read-only, from 0 to 0xF00).",
	},
	{
		"CT",
		"cmd-mode-timeout",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Command Mode Timeout: the period of inactivity (no valid "
		"commands received) after which the RF module automatically "
		"exits AT Command Mode and returns to Idle Mode (2 to 0x1770, "
		"default is 0x64).",
	},
	{
		"CN",
		"exit-cmd-mode",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Exit Command Mode.",
	},
	{
		"GT",
		"guard-times",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Guard Times: period of silence in ms before and after the "
		"Command Sequence Characters of the AT Command Mode Sequence, "
		"used to prevent inadvertent entrance into AT Command Mode "
		"(0 to 0xFFFF, default is 0x3E8).",
	},
	{
		"CC",
		"command-chars",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Command Character used between guard times of the AT Command "
		"Mode Sequence (0 to 0xFF, default is 0x2B).",
	},
	{
		"ID",
		"network-id",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Network ID. Nodes must have the same network identifier "
		"to communicate (0 to 0x7FFF, default is 0x7FFF).",
	},
	{
		"NT",
		"ndisc-timeout",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Node Discover Timeout, time in tenth of secs a node will "
		"spend discovering other nodes when ND or DN is issued (0 "
		"to 0xFC, default is 0x82).",
	},
	{
		"NI",
		"node-id",
		XBEE_ATCMD_F_PARAM_STRING_20B | XBEE_ATCMD_F_READ |
		XBEE_ATCMD_F_WRITE,
		"Node Identifier in printable ASCII characters. This string is "
		"returned as part of the ATND (Network Discover) command. This "
		"identifier is also used with the ATDN (Destination Node) "
		"command. The string contains up to 20 byte ASCII string, "
		"default is a space character.",
	},
	{
		"DN",
		"disc-node",
		XBEE_ATCMD_F_PARAM_STRING_20B | XBEE_ATCMD_F_READ |
		XBEE_ATCMD_F_WRITE,
		/* XXX */
		"Resolves a Node Identifier string to a physical address "
		"(case sensitive). 0xFFFE and the 64bits extended address are "
		"returned."
	},
	{
		"ND",
		"network-discover",
		XBEE_ATCMD_F_PARAM_NONE | XBEE_ATCMD_F_WRITE,
		"Network Discovery, see doc", /* XXX */
	},
	{
		"NO",
		"ndisc-options",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Network Discovery Options, a bitfield value that changes the "
		"behavior of the ND command (bit0 - Append DD value, bit1 - "
		"Local device sends ND response frame when ND is issued). "
		"Default is 0."
	},
	{
		"EE",
		"security-enable",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Enable or disable 128-bit AES encryption (0 or 1, 0 is the "
		"default).",
	},
	{
		"KY", /* XXX */
		"security-key",
		XBEE_ATCMD_F_PARAM_HEXBUF_16B | XBEE_ATCMD_F_WRITE,
		"The 128bits security key (the command is write-only).",
	},
	{
		"MT",
		"bcast-multi-xmit",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Number of additional MAC-level broadcast transmissions. All "
		"broadcast packets are transmitted MT+1 times to ensure "
		"it is received (0 to 0xF, default is 3).",
	},
	{
		"RR",
		"unicast-retries",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Number of additional MAC-level packet delivery attempts for "
		"unicast transactions. If RR is non-zero, packets sent from "
		"the radio will request an acknowledgement, and can be resent "
		"up to RR times if no acknowledgement is received. (0 to 0xF, "
		"default is 10).",
	},
	{
		"PL",
		"power-level",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Power Level of RF transmitter (0 - 1mW, 1 - 23mW, 2 - 100mW, "
		"3 - 158 mW, 4 - 316 mW). Default is 4.",
	},
	{
		"SM",
		"sleep-mode",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Sleep Mode (0 - disabled, 1 - pin sleep, 4 - async cyclic "
		"sleep, 5 - async cyclic sleep with pin wakeup). Default "
		"is 0.",
	},
	{
		"SO",
		"sleep-options",
		XBEE_ATCMD_F_PARAM_U8 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Sleep Options bitmask (bit8 - always wake for ST time). "
		"Default is 0.",
	},
	{
		"ST",
		"wake-time",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Wake Time: the amount of time in ms that the module will stay "
		"awake after receiving RF or serial data (from 0x45 to "
		"0x36EE80, default is 0x7D0 = 2 secs).",
	},
	{
		"SP",
		"sleep-period",
		XBEE_ATCMD_F_PARAM_U32 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Sleep Period: the amount of time in 10ms unit the module will "
		"sleep per cycle.  For a node operating as an Indirect "
		"Messaging Coordinator, this command defines the amount of "
		"time that it will hold an indirect message for an end device. "
		"The coordinator will hold the message for (2.5 * SP). Range "
		"is from 1 to 1440000, default is 200 (2 secs).",
	},
	{
		"SN",
		"num-sleep-periods",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Number of Sleep Periods that must elapse between assertions "
		"of the ON_SLEEP line during the wake time of asynchronous "
		"cyclic sleep (1 to 0xFFFF, default is 1).",
	},
	{
		"WH",
		"wake-host",
		XBEE_ATCMD_F_PARAM_U16 | XBEE_ATCMD_F_READ | XBEE_ATCMD_F_WRITE,
		"Wake Host time. If it is set to a non-zero value, it "
		"specifies the time in ms that the device should allow after "
		"waking from sleep before sending data out the UART or "
		"transmitting an I/O sample. If serial characters are "
		"received, the WH timer is stopped immediately. Range is "
		"from 0 to 0xFFFF, default is 0.",
	},
	{
		NULL,
		NULL,
		0,
		NULL,
	},
};

struct xbee_atcmd *xbee_atcmd_lookup_name(const char *atcmd_str)
{
	struct xbee_atcmd *cmd;

	for (cmd = &xbee_atcmd_list[0]; cmd->name != NULL; cmd++) {
		if (strcmp(atcmd_str, cmd->name))
			continue;
		break;
	}

	if (cmd->name == NULL) /* not found */
		return NULL;

	return cmd;
}

struct xbee_atcmd *xbee_atcmd_lookup_desc(const char *desc)
{
	struct xbee_atcmd *cmd;

	for (cmd = &xbee_atcmd_list[0]; cmd->desc != NULL; cmd++) {
		if (strcmp(desc, cmd->desc))
			continue;
		break;
	}

	if (cmd->name == NULL) /* not found */
		return NULL;

	return cmd;
}
