/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/queue.h>

#include "xbee_neighbor.h"
#include "xbee_stats.h"
#include "xbee_proto.h"
#include "xbee_buf.h"
#include "xbee.h"

struct xbee_stats *xbee_get_stats(struct xbee_dev *dev)
{
	return &dev->stats;
}

void xbee_reset_stats(struct xbee_dev *dev)
{
	memset(&dev->stats, 0, sizeof(dev->stats));
}

void xbee_dump_stats(FILE *file, struct xbee_dev *dev)
{
	fprintf(file, "statistics on dev %s:\n", dev->name);
	fprintf(file, " rx_frame: %d\n", dev->stats.rx_frame);
	fprintf(file, " rx_atresp: %d\n", dev->stats.rx_atresp);
	fprintf(file, " rx_atresp_error: %d\n", dev->stats.rx_atresp_error);
	fprintf(file, " rx_modem_status: %d\n", dev->stats.rx_modem_status);
	fprintf(file, " rx_xmit_status: %d\n", dev->stats.rx_xmit_status);
	fprintf(file, " rx_xmit_status_error: %d\n", dev->stats.rx_xmit_status_error);
	fprintf(file, " rx_data: %d\n", dev->stats.rx_data);
	fprintf(file, " rx_expl_data: %d\n", dev->stats.rx_expl_data);
	fprintf(file, " rx_node_id: %d\n", dev->stats.rx_node_id);
	fprintf(file, " rx_rmt_atresp: %d\n", dev->stats.rx_rmt_atresp);
	fprintf(file, " rx_rmt_atresp_error: %d\n", dev->stats.rx_rmt_atresp_error);
	fprintf(file, " rx_frame_too_small: %d\n", dev->stats.rx_frame_too_small);
	fprintf(file, " rx_frame_too_large: %d\n", dev->stats.rx_frame_too_large);
	fprintf(file, " rx_invalid_cksum: %d\n", dev->stats.rx_invalid_cksum);
	fprintf(file, " rx_invalid_type: %d\n", dev->stats.rx_invalid_type);
	fprintf(file, " rx_no_delim: %d\n", dev->stats.rx_no_delim);
	fprintf(file, " tx_frame: %d\n", dev->stats.tx_frame);
	fprintf(file, " tx_atcmd: %d\n", dev->stats.tx_atcmd);
	fprintf(file, " tx_atcmd_q: %d\n", dev->stats.tx_atcmd_q);
	fprintf(file, " tx_data: %d\n", dev->stats.tx_data);
	fprintf(file, " tx_expl_data: %d\n", dev->stats.tx_expl_data);
	fprintf(file, " tx_xmit_retries: %d\n", dev->stats.tx_xmit_retries);
	fprintf(file, " tx_rmt_atcmd: %d\n", dev->stats.tx_rmt_atcmd);
	fprintf(file, " tx_invalid_type: %d\n", dev->stats.tx_invalid_type);
	fprintf(file, " tx_invalid_channel: %d\n", dev->stats.tx_invalid_channel);
}
